/*
   piktc.h -- PIKT common header file for piktc & piktc_svc

   $Header: /var/cvs/PIKT/pikt/src/piktc/piktc.h,v 1.68 2007/09/06 21:34:24 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2007 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define MODE_PRG_DEFAULT	"750"	/* default mode for installed
					   programs
					 */
#define MODE_FIL_DEFAULT	"640"	/* default mode for installed
					   files
					 */
#define UID_FIL_DEFAULT		"0"	/* default uid for installed
					   programs/files
					 */
#define GID_FIL_DEFAULT		"0"	/* default gid for installed
					   programs/files
					 */

/* additional rpc request states */
#define DELETE		0
#define ADD		1
#define EXECUTE		2
#define EXECUTEWAIT	3

/* the order of these is vitally important! */
#define OPS		0
#define HST		1
#define GRP		2
#define ALS		3
#define DEF		4
#define MCR		5
#define ALM		6
#define ALT		7
#define PRG		8
#define FIL		9
#define OBJ		10

/* the order of these is vitally important! */
#define SYSCFG		0
#define DEFCFG		1
#define MCRCFG		2
#define ALMCFG		3
#define ALTCFG		4
#define PRGCFG		5
#define FILCFG		6
#define OBJCFG		7

/* #include file type */
#define INCFILE		0
#define INCPROC		1

#define MCRLNK	'\001'	/* the char linking a macroname to its
			   macro prm or macro num
			 */

/* used to record command-line alert/program/file/object/host names;
   alert/program/file/object names in cfg.c
 */
typedef struct _nam {
	char *name;
	char *aux;		/* only used by alerts/commands */
	char *dir;		/* only used by programs/files for dir */
	char *file;		/* only used by programs/files for file */
	char *mode;		/* only used by programs/files for mode */
	char *filuid;		/* only used by programs/files for uid */
	char *filgid;		/* only used by programs/files for gid */
	int state;		/* ADD/DELETE/ERROR/IGNORE */
	struct _nam *next;
} nam;

typedef struct {
	unsigned int len;
	char *val;
} txt;

/* used when passing service requests registrations to piktc_svc
 */
typedef struct {
	char *hostname;		/* name of the remote piktc_svc host */
	char *request;		/* the requested service */
	int state;		/* the service state */
	int enctyp;
	txt keysum;		/* key checksum */
} req;

/* used when passing files from piktc to piktc_svc
 */
typedef struct {
	char *dir;		/* file directory */
	char *file;		/* filename */
	char *ext;		/* file extension, e.g., .alt */
	char *mode;		/* file permissions */
	char *filuid;		/* file uid */
	char *filgid;		/* file gid */
	txt text;		/* file contents as a string val of len bytes */
	unsigned int len;	/* file length in bytes */
	int enctyp;
	txt keysum;		/* key checksum */
	int backup;		/* do backup? */
} fil;

/* used when passing non-file items from piktc to piktc_svc
*/
typedef struct {
	char *name;
	char *aux;
	int state;		/* ADD/DELETE/ERROR/IGNORE */
	int enctyp;
	txt keysum;		/* key checksum */
} itm;

/* used to store lines from all .cfg files */
typedef struct _cfgln {
	int lineno;		/* line number */
	char *file;		/* line's cfg file */
	unsigned long *bs;	/* line's host bitset "tag" */
	char *line;		/* line text */
	int len;		/* line length */
	int domcr;		/* macro expand this line? */
	int doproc;		/* process this line? */
	struct _cfgln *next;
} cfgln;

typedef struct _def {
	char *component;	/* macro/alarm def component */
	struct _def *next;
} def;

/* note that not all fields are used for every ident */
typedef struct _ident {
	char *name;		/* ident name */
	int type;		/* DEF/OPS/HST/GRP/ALS/MCR/ALM/ALT/PRG/FIL/OBJ */
	char *aux;		/* only used by ALS & ALT types */
	unsigned long *bs;	/* ident's host/define bitset "tag" */
	def *definition;	/* list of ident defs */
	def *defcurr;		/* current definition */
	int prmnum;		/* number of macro parameters */
	int called;		/* are we in the middle of a macro call
				   by this name?
				 */
	int piktuid;		/* for a host, its unique auth piktuid */
	int piktgid;		/* for a host, its unique auth piktgid */
	char *piktprivate_key;	/* for a host, its unique auth private_key */
	int piktdataenctyp;	/* for a host, its data encryption type */
	int piktauthenctyp;	/* for a host, its auth encryption type */
	struct _ident *next;
} ident;

extern const char piktc_version[];
extern const char piktc_svc_version[];

extern int havekeysconf;

extern AUTHUNIX_CREATE_UID_T authuidmaster;	/* piktmaster authuid */
extern AUTHUNIX_CREATE_GID_T authgidmaster;	/* piktmaster authgid */
extern char *pvtkeymaster;			/* piktmaster private
						   encryption key
						 */
extern int dataenctypmaster;			/* piktmaster data
						   encryption type
						 */
extern int authenctypmaster;			/* piktmaster auth
						   encryption type
						 */

extern int cfgerr;	/* signifies syntax error in any config file */
extern int stzerr;	/* signifies syntax error in current stanza */

extern nam *commands,               *incmdcommands,                 *cmdcommands,
           *alerts,   *cfgalerts,   *incmdalerts,   *excmdalerts,   *cmdalerts,
           *programs, *cfgprograms, *incmdprograms, *excmdprograms, *cmdprograms,
           *files,    *cfgfiles,    *incmdfiles,    *excmdfiles,    *cmdfiles,
           *objects,  *cfgobjects,  *incmdobjects,  *excmdobjects,  *cmdobjects,
                      *cfghosts,    *incmdhosts,    *excmdhosts,    *cmdhosts;

extern int commandsall, alertsall, programsall,	   /* was +? "all" specified */
           filesall, objectsall, hostsall;	   /* on commandline?        */

extern int commandsnone, alertsnone, programsnone, /* was -? "all" specified */
           filesnone, objectsnone, hostsnone;	   /* on commandline?        */

extern int havecmditems;	/* were any of +/- A, +/-P, +/-F, +/-O, or
				   +/-C specified on commandline?
				 */

extern int havecmdmacros[OBJ+1]; /* were any item macros for A, P, F or O
				    specified on commandline?
				  */

extern int havecmdhosts;	/* was +/-H specified on commandline? */

extern FILE	*stripin,	*stripout,
		*tagin,		*tagout,
		*sysin,		*sysout,
		*cfgin,		*cfgout;

extern char *infile;		/* current input infile */
extern char *inproc;		/* current input process */

extern char *inctmpfile;	/* temporary include file */
extern FILE *inctmpout;		/* temporary include file handle */
extern int incdevnull;		/* are we #include'ing /dev/null? */

extern int fd[];		/* channel from striplex() to *parse() */
extern pid_t pid;		/* pid of current child; for striplex() */

extern int **p2cfd, **c2pfd;	/* ipc parent-to-child and
				   child-to-parent channels;
				   for coordinating striplex()
				   with *parse()
				 */

extern const char *cfgfilenames[];	/* systems.cfg/defines.cfg/macros.cfg/
					   alarms.cfg/alerts.cfg/programs.cfg/
					   files.cfg/objects.cfg
					 */
extern char *cfgfileprev;	/* name of previous config file */
extern char *cfgfilecurr;	/* name of current config file */

extern const char *cfgbof[];	/* signifies beginning of new config file */
extern char *cfgline;

extern int cfgcurr;		/* current config file */
extern int cfglast;		/* last config file to be processed */
extern cfgln *configs[];	/* array of pointers to cfg file lines */
extern cfgln *clplast;		/* pointer to last cfg line */

extern int indented;		/* are we in #indent mode? */
extern int verbatim;		/* are we in #verbatim mode? */

extern ident *idents[];			/* ident hash table */
extern const char *identypes[];		/* OPS/HST/GRP/ALS/DEF/MCR/ALM/ALT/PRG/FIL/OBJ */

extern unsigned long  cfbs;		/* actual config bitset */
extern unsigned long rcfbs;		/* required config bitset */
extern unsigned long mcfbs;		/* missing config bitset */

extern size_t hbslongs;			/* number of longs in a host bit-set */
extern size_t dbslongs;			/* number of longs in a define bit-set */

extern unsigned long *ghbs;		/* global host bitset */

extern unsigned long *hbsfull, *hbsempty; /* full and empty host bitsets */

extern unsigned long *hbscurr;		/* host bitset for the current
					     piktc command
					 */

extern unsigned long *gdbs;		/* global define bitset */
extern unsigned long *gubs;		/* global undefined bitset */

extern unsigned long *dbsfull, *dbsempty; /* full and empty define bitsets;
					     currently serve no purpose
					   */

extern unsigned long *dbscurr;		/* define bitset for the current
					   piktc command
					 */

extern char **hosts;			/* system hostnames */
extern int hostnum;			/* the number of configured hosts */
extern char *hostcurr;			/* the current hostname */

extern char **opsys;			/* operating systems */
extern int opsysnum;			/* the number of configured os'es */

extern char **hstgrp;			/* host groups */
extern int hstgrpnum;			/* the number of configured host
					   groups
					 */

extern int defnum;			/* the number of configured defines */

extern const char piktnullchar[];	/* for =piktnullchar definition */
extern const char piktspacechar[];	/* for =piktspacechar definition */
extern const char pikttabchar[];	/* for =pikttabchar definition */
extern const char piktnewlinechar[];	/* for =piktnewlinechar definition */

extern unsigned loffset;		/* offset from beginning of line */
extern unsigned bof;			/* beginning of file */
extern unsigned bon;			/* beginning of (line) number */
extern unsigned bol;			/* beginning of line */

extern int timings;				/* the number of different
						   timings for the alert
						 */
extern char *mins[], *hrs[],			/* timing parameters */
            *doms[], *mons[],
            *dows[];
extern char *drift[];				/* random drifts (in mins)
						   from the alert timings
						 */
extern char *priority;				/* alert nice level */
extern char *nicecmd, *mailcmd, *lpcmd;		/* strings written to
						   piktd.conf
						 */
extern char *execcmd;				/* for script one-liners
						   specified in alerts.cfg
						 */

extern int stripalm, stripprg, stripfil, stripobj;	/* strip these files
							   of comments?
							 */

extern int hasmacro;			/* does line appear to have a macro? */
extern int domacros;			/* do macro processing? */
extern int dostrip;			/* do comment stripping? */

extern int incnum;			/* the current (nested) include
					   file number
					 */

extern int inctype;			/* the current #include file type */

extern char *hostserv;			/* piktc_svc service host */
extern char *rqstserv;			/* callback service request */
extern int statserv;			/* piktc_svc service state */

extern const char *transports[];	/* "udp" or "tcp" */

extern char *key;			/* encryption key */
extern unsigned int keylen;		/* encryption key length */
extern char *keyenc;			/* encrypted key */
extern char *keysum;			/* key checksum */

/* init functions */
extern int chkpiktc_svc(char *host);
extern int piktc_svc_main(void);

/* command-line functions */
extern void piktc_usage(void);
extern void piktc_svc_usage(void);
extern void preprocesscmd(void);
extern nam *regname(nam **, char *, int);
extern void retire(nam **);
extern nam *lookupname(nam *, char *);
extern nam *purge(nam *, nam *, nam *, int);

/* lexer and parser functions */
extern int cmdparse(void);
extern int sysparse(void);
extern int tagparse(void);
extern int defparse(void);
extern int cfgparse(void);
extern int keysparse(void);
extern int cmdlex(void);
extern int syslex(void);
extern int taglex(void);
extern int deflex(void);
extern int cfglex(void);
extern int keyslex(void);
extern void cmderror(const char *);
extern void syserror(const char *);
extern void tagerror(const char *);
extern void deferror(const char *);
extern void cfgerror(const char *);
extern void keyserror(const char *);
extern int striplex(void);
extern void inspectline (char *, int *, int);
extern void endsysincfile(void);
extern void endtagincfile(void);

/* include functions */
extern FILE *instrmopen(int, char *, char *, int, char *, int);
extern int instrmclose(int, FILE *, int, char *, int);
extern int isverbatim(char *);
extern void check_incfile(int, char *);
extern int create_incfile(char *);

/* macro functions */
extern char *parametize(char *, unsigned int, char *, int);
extern char *expand(char *, int);
extern char *mcrexpand(char *, int, char *, int, int);
extern void setmcrargs(char *, int, int, char *, int);
extern void unsetmcrargs(char *, int, int, int);
extern char *dynmcrexpand(char *, char *, char *, char *, char *);

/* ident functions */
extern ident *lookupident(char *, int);
extern ident *regident(char *, int, int);
void cleanup(void);
extern void makepiktmacros(char *, int);
extern void makepiktdefs(void);
extern char *getnicecmd(char *);
/* extern void makenongroups(char **, int); */
extern void makenongroup(char *);

/* bitset functions */
extern void empty(unsigned long *, size_t);
extern void fill(unsigned long *, size_t);
extern unsigned long *bsunion(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsintersection(unsigned long *, unsigned long *, size_t);
extern unsigned long *bscomplement(unsigned long *, size_t);
extern int bsmember(unsigned long *, unsigned long *, size_t);
extern int bssubset(unsigned long *, unsigned long *, size_t);
extern void bscopy(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsdup(unsigned long *, size_t);
extern unsigned long *getbs(char *, int, int);
extern void tag(int, char *, unsigned long *, size_t, int, int, char *);
extern ident *insert(char *, int);

/* alt/prg/fil/obj file functions */
extern void regcomponent(ident *, char *);
extern void beginalarms(char *, int);
extern void writealarm(char *, char *, char *, char *);
extern void endalarms(char *);
extern void beginfile(char *, char *, char *, char *, char *, nam **);
extern void writefile(char *);
extern void endfile(char *);
extern void beginobjects(char *);
extern void writeobject(char *);
extern void endobjects(char *);
extern int piktdconf_toolong(char *, int);
extern void mkdirtree(char *);

/* rpc functions */
extern void enable_alerts(const char *, const nam *, const int);
extern void disable_alerts(const char *, const nam *, const int);
extern void execute_commands(const char *, const nam *, const int);
extern void execute_alerts(const char *, const nam *, const int);
extern void expand_macros(const char *, const nam *, const int);
extern void install_files(const char *, const nam *, const char *, const char *, const char *, const char *, const char *, const int, const int);
extern void fetch_files(const char *, const nam *, const char *, const char *, const int);
extern void cksum_files(const char *, const nam *, const char *, const char *, const int, const int);
extern void delete_files(const char *, const nam *, const char *, const char *, const int, const int);
extern void restore_files(const char *, const nam *, const char *, const char *, const int);
extern void view_files(const char *, const nam *, const char *, const char *, const int, const int);
extern void list_files(const char *, const nam *, const char *, const char *, const int);
extern void list_items(const char *, const nam *, const char *, const char *, const int);
extern void install_piktconf(const char *, const char *, const char *, const char *, const int, const int);
extern void kill_daemon(const char *, const unsigned long, const int);
extern void restart_daemon(const char *, const unsigned long, const int);
extern void show_alerts_stata(const char *, const nam *, const int);
extern void register_request(const char *, const char *, const char *, const int);
extern int call_back(const char *, const char *, const int);

/* debugging functions */
extern void dumpcfgs(unsigned long *);
